/**
 * This software is provided "AS IS," without a warranty of any kind.
 * By using it you are agreeing to not held the author responsible for any damage.
 */
package jqb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.SASLAuthentication;
import org.jivesoftware.smack.SmackConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smackx.muc.DiscussionHistory;
import org.jivesoftware.smackx.muc.MultiUserChat;
/**
 * The Mainclass, which does everything
 * @author wolf.posdorfer
 *
 */
public class Main
{
	private String SERVER;
	private String ROOM;
	private String USERNAME;
	private String PASSWORD;
	private String BOTNAME;
	private int TIMERCOUNT;

	private XMPPConnection _connection;
	private Message _lastmessage;
	private String _owner;
	private ArrayList<Question> _questions;
	private ArrayList<Spieler> _spieler;
	private boolean _quizactive;
	private Question _currentQuestion;
	private MultiUserChat _muc;
	private int _timercount;
	private Timer _timer;
	private boolean _timeractive;

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{

		if (args.length > 0 && args[0].compareToIgnoreCase("-gui") == 0)
		{
			MainGui mg = new MainGui();
			mg.setVisible(true);
		}
		else
		{
			new Main();
		}
	}

	public Main()
	{

		_questions = new ArrayList<Question>();
		_spieler = new ArrayList<Spieler>();
		_quizactive = false;
		_lastmessage = new Message();
		_lastmessage.setBody("");
		_currentQuestion = new Question("ababababababa", "abababababab", "ababababababab",
				123123123);
		_timer = new Timer();
		_timeractive = false;

		TextFileParser tfp = new TextFileParser();
		_questions = tfp.gibFragen();
		if (_questions.size() <= 0)
		{
			System.err.println("Fragen Datenbank ist leer!");
		}
		_spieler = tfp.gibSpieler();

		String[] login = tfp.gibLoginInfo();

		SERVER = login[0];
		ROOM = login[1];
		USERNAME = login[2];
		PASSWORD = login[3];
		BOTNAME = login[4];
		TIMERCOUNT = Integer.valueOf(login[5]);

		ConnectionConfiguration config = new ConnectionConfiguration(SERVER, 5222);
		_connection = createcon(config);

		_owner = USERNAME;

		while (!_connection.isConnected())
		{
			try
			{
				Thread.sleep(100);
			}
			catch (InterruptedException e)
			{
				System.err.println(e.getMessage());
			}
		}
		login(_connection, USERNAME, PASSWORD);
		joinChatroom();

		while (!_lastmessage.getBody().equalsIgnoreCase("!quit"))
		{
			while (_quizactive && _timeractive)
			{

				if (Antwort.checkAnswer(_currentQuestion.gibAntwort(), _lastmessage.getBody()))
				{
					rewardWinner(_lastmessage);
					_timercount = 0;
					_timer.cancel();
					sendQuestion();
				}
			}
		}

		disconnect(_connection);
	}

	/**
	 * Give Points to the Winner
	 * @param message
	 */
	private void rewardWinner(Message message)
	{

		String spielername = message.getFrom().replace(ROOM + "/", "");
		boolean spielerfound = false;

		String rewardmessage = "\nLösung: " + _currentQuestion.gibAntwort();  // The Answer :  <answer>

		for (Spieler s : _spieler)
		{
			if (spielername.equalsIgnoreCase(s.gibName()))
			{
				if (!spielerfound)
				{
					s.addPunkte(1);
					sendMessage(_muc, spielername + " erhält 1 Punkt für die Lösung"
							+ rewardmessage);  // <playername> gets 1 point for the answer : <answer>
					spielerfound = true;

				}
			}
		}

		if (!spielerfound)
		{
			_spieler.add(new Spieler(spielername, 1));
			sendMessage(_muc, spielername + " erhält seinen ersten Punkt für die Lösung"
					+ rewardmessage);  // <playername> gets his first point for the answer : <answer>
		}

		Collections.sort(_spieler);
		TextFileWriter.writeScoreBoard(_spieler);

	}

	/**
	 * starts connection
	 * 
	 * @return
	 */
	private XMPPConnection createcon(ConnectionConfiguration config)
	{
		XMPPConnection conn1 = new XMPPConnection(config);
		try
		{
			conn1.connect();
			System.out.println("Connection Successfull");
			SASLAuthentication.supportSASLMechanism("PLAIN", 0);
		}
		catch (XMPPException e)
		{
			System.err.println(e.toString());
		}
		return conn1;
	}

	/**
	 * Disconnects the Bot
	 * @param con
	 */
	public void disconnect(XMPPConnection con)
	{
		sendMessage(_muc, "Tschüssikofski!!");  // Ciao, byebye
		con.disconnect();
		_timer.cancel();
		System.out.print("Speichere Scoreboad...."); // Saving Scoreboard....
		TextFileWriter.writeScoreBoard(_spieler);
		System.out.print("....Fertig.\n"); // .... Done

		System.err.println("shutting down!");
		System.exit(0);
	}

	/**
	 * Login
	 * 
	 * @param connection
	 * @param username
	 * @param password
	 */
	private void login(XMPPConnection connection, String username, String password)
	{
		try
		{
			connection.login(username, password, BOTNAME);
			System.out.println("Logged in Successfull to " + connection.getHost() + " as "
					+ BOTNAME);
		}
		catch (XMPPException e)
		{
			System.err.println(e.toString());
		}
	}

	private void joinChatroom()
	{
		_muc = new MultiUserChat(_connection, ROOM);

		try
		{

			DiscussionHistory history = new DiscussionHistory();
			history.setMaxStanzas(0);
			_muc.join(BOTNAME, "", history, SmackConfiguration.getPacketReplyTimeout());
			// Iterator<String> arlist = muc.getOccupants();

		}
		catch (XMPPException e1)
		{
			e1.printStackTrace();
		}

		final MultiUserChat muc2 = _muc;

		muc2.addMessageListener(new PacketListener()
		{

			public void processPacket(Packet arg0)
			{
				Message m = (Message) arg0;
				System.out.println(m.getFrom().replace(ROOM + "/", "") + ": " + m.getBody());

				handleMessages(muc2, m);
			}
		});
	}

	/**
	 * Handles messages, Check if it is a ChatBot command or a answer guess
	 * @param muc
	 * @param m
	 */
	public void handleMessages(MultiUserChat muc, Message m)
	{

		String getFrom = m.getFrom().replace(ROOM + "/", "");

		if (m.getBody().contains("!help") && !getFrom.equals(BOTNAME))
		{
			handleHelpMessage(muc, m);

		}
		else if (m.getBody().equalsIgnoreCase("!quit")
				&& (getFrom.contains(_owner) || getFrom.contains(BOTNAME)))
		{
			_lastmessage.setBody("!quit");
		}
		else if (m.getBody().contains("!quiz") && !getFrom.equals(BOTNAME))
		{
			if (m.getBody().length() <= 6)
			{
				Message z = new Message();
				z.addBody("de", "!quiz t");
				handleQuizMessage(muc, z, getFrom);
			}
			else
			{

				handleQuizMessage(muc, m, getFrom);
			}
		}
		else
		{
			if (_quizactive)
			{
				_lastmessage = m;
			}

		}
	}

	private void handleHelpMessage(MultiUserChat muc, Message m)
	{
		if (m.getBody().equalsIgnoreCase("!help"))
		{
			String hilfetext = "Ich beherrsche folgende Befehle:\n" + "!help !quiz !quit";  // I know the following Commands: !help !quiz !quit
			_lastmessage = m;
			sendMessage(muc, hilfetext);
		}

		String split = m.getBody().split(" ")[1];

		if (split.equals("quiz"))
		{
			sendMessage(muc, "Für !quiz gibt es folgende Zusatzparameter:\nstart stop top5 rank"); // the following parameters must be added to !quiz: start stop top5 rank
		}
		else
		{
			sendMessage(muc, "Ich weiß nicht was du meinst."); // I have no idea what youre talking about
		}
	}

	private void handleQuizMessage(MultiUserChat muc, Message m, String getFrom)
	{

		String s = m.getBody().split(" ")[1];

		if (s.equalsIgnoreCase("start"))
		{
			// start quiz
			if (getFrom.contains(USERNAME))
			{
				_quizactive = true;
				sendQuestion();
			}
		}
		else if (s.equalsIgnoreCase("stop"))
		{
			// stop quiz
			if (getFrom.contains(USERNAME))
			{
				_quizactive = false;
				_timer.cancel();
				_timeractive = false;
				_timercount = 0;
				sendMessage(muc, "Okay dann eben kein Quiz mehr...."); // Okay no more quiz
				System.out.println("Quiz deaktiviert");  // Deactivated Quiz
			}
		}
		else if (s.equalsIgnoreCase("top5"))
		{
			sendQuizScoreboard(muc);
		}
		else if (s.equalsIgnoreCase("rank"))
		{
			sendQuizRank(muc, getFrom);
		}
		else
		{
			sendMessage(muc,
					"Keine Ahnung was du meinst...\n Befehle sind: start, stop, top5, rank");   // No idea what youre talking about commands are: start....
		}
	}

	private void sendQuizRank(MultiUserChat muc, String getFrom)
	{
		for (Spieler spieler : _spieler)
		{
			if (spieler.gibName().equalsIgnoreCase(getFrom))
			{
				sendMessage(muc, "Du bist auf Platz:" + (1 + _spieler.indexOf(spieler)) + " mit "
						+ spieler.gibPunkte() + " Punkten");  // you are on the <place>Place with <points> Points
			}
		}
	}

	private void sendQuizScoreboard(MultiUserChat muc)
	{
		int end = 5;
		if (_spieler.size() < 5)
			end = _spieler.size();

		String scorelist = "Die Top " + end + " sind:\n";  // The top <end> are:

		for (int i = 0; i < end; i++)
		{
			scorelist += (i + 1) + ". " + _spieler.get(i).gibName() + " - "
					+ _spieler.get(i).gibPunkte() + "pts\n";  // <place>. <playername> - <points> pts
		}
		sendMessage(muc, scorelist);
	}

	/**
	 * sendet eine Message im angegebenen MUC
	 * 
	 * @param muc
	 * @param message
	 */
	public void sendMessage(MultiUserChat muc, String message)
	{
		try
		{
			muc.sendMessage(message);
		}
		catch (XMPPException e)
		{
			System.err.println(e.getMessage());
		}
	}

	public void sendQuestion()
	{

		try
		{
			Thread.sleep(3000);
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		int random = 0;
		Random rng = new Random();
		random = rng.nextInt(_questions.size());
		_currentQuestion = _questions.get(random);
		String fragestellen = TIMERCOUNT + " Sekunden zur Beantwortung\n" + "Kategorie: "
				+ _currentQuestion.gibCategory() + "\n\n" + _currentQuestion.gibFrage(); // <Timer> seconds for answer, \n Category: <category>
		sendMessage(_muc, fragestellen);

		System.out.println(">>>>> Antwort: " + _currentQuestion.gibAntwort()); // >>>> answer : <Answer>
		_timercount = TIMERCOUNT;
		startTimer();

	}

	public void startTimer()
	{
		_timeractive = true;
		_timer = new Timer();
		TimerTask task;
		task = new TimerTask()
		{
			@Override
			public void run()
			{
				if (_timercount == 1)
				{
					_timer.cancel();
					_timeractive = false;
					_timercount = 0;

					String zeitabgelaufen = "Zeit abgelaufen\nDie Antwort war: "
							+ _currentQuestion.gibAntwort(); // Time's up the answer was: <answer>
					sendMessage(_muc, zeitabgelaufen);
					sendQuestion();
				}
				else if (_timercount > 1)
				{
					_timercount -= 1;
				}
				else
				{
					_timer.cancel();
					_timeractive = false;
				}
			}
		};
		_timer.schedule(task, 1000, 1000);
	}

}
