/**
 * This software is provided "AS IS," without a warranty of any kind.
 * By using it you are agreeing to not held the author responsible for any damage.
 */
package jqb;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

/**
 * The Textfile Parser used for parsing the questions, scoreboard and login information
 * @author wolf.posdorfer
 * 
 */
public class TextFileParser
{

	private final String DATENBANK = "./questiondb.txt";
	private final String SCOREBOARD = "./scoreboard.txt";
	private final String LOGIN = "./login.txt";

	private ArrayList<Question> _questions;
	private ArrayList<Spieler> _spieler;
	private String[] _logininfo;

	public TextFileParser()
	{
		_questions = new ArrayList<Question>();
		_spieler = new ArrayList<Spieler>();
		_logininfo = new String[6];

		try
		{
			readlogin();
			readDB();
			readScoreBoard();
			Collections.sort(_spieler);
		}
		catch (IOException e)
		{
			System.err.print(e.getMessage());
		}
	}

	private void readlogin() throws IOException
	{
		File file = new File(LOGIN);
		if (file.exists())
		{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String zeile = "";
			int i = 0;
			while ((zeile = reader.readLine()) != null)
			{
				zeile = zeile.split("=")[1];
				_logininfo[i] = zeile;
				i++;
			}
			System.out.println("Login info read.");
		}
		else
		{
			System.err.println("File not found, Creating Basic info.txt");
			file.createNewFile();
			
			FileWriter f;
			try
			{
				f = new FileWriter(LOGIN, false);
				f.write("server=myserver.de\nroom=room@myserver.de\nuser=myuser\npass=mypassword\nnick=QuizBot\nquestionduration=45");
				f.close();
			}
			catch (IOException e)
			{
				System.err.println(e.getMessage());
			}

			System.err.println("shutting down.");
			System.exit(0);
		}

	}

	private void readScoreBoard() throws IOException
	{
		File file = new File(SCOREBOARD);

		if (file.exists())
		{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String zeile = "";
			while ((zeile = reader.readLine()) != null)
			{
				String name = StringZuWerte(zeile)[0];
				String punkte = StringZuWerte(zeile)[1];
				_spieler.add(new Spieler(name, Integer.parseInt(punkte)));
			}

			Collections.sort(_spieler);
			System.out.println("Done Parsing! Parsed " + _spieler.size() + " Players and Scores");
		}
		else
		{
			System.err.println("File not found, Creating Scoreboard");
			file.createNewFile();
		}
	}

	public ArrayList<Question> gibFragen()
	{
		return _questions;
	}

	public ArrayList<Spieler> gibSpieler()
	{
		return _spieler;
	}
	public String[] gibLoginInfo()
	{
		return _logininfo;
	}

	public void readDB() throws IOException
	{

		File file = new File(DATENBANK);
		BufferedReader reader = new BufferedReader(new FileReader(file));

		String zeile = "";
		int id = 0;
		while ((zeile = reader.readLine()) != null)
		{
			String category = StringZuWerte(zeile)[1];
			String antwort = StringZuWerte(zeile)[2];

			String frage = StringZuWerte(zeile)[3];

			_questions.add(new Question(frage, antwort, category, id));

			id++;

		}

		System.out.println("Done Parsing! Parsed " + id + " Questions");

		// System.out.println(_questions.get(0).gibCategory() +
		// _questions.get(0).gibAntwort()
		// + _questions.get(0).gibFrage());
	}

	/**
	 * Teil einen Datenbank String auf und wandelt ihn in einen Array um
	 * 
	 * @param s
	 * @return
	 */
	public String[] StringZuWerte(String s)
	{
		String regex = ";";
		String[] stringarr = s.split(regex);
		return stringarr;
	}

}
